<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.1
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * Model
 * 
 * Каркас для модели модуля
 */
abstract class Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var Init основной объект системы
	 */
	protected $diafan;

	/**
	 * @var string текущий модуль, для котого вызвана шаблонная функция
	 */
	private $current_module;

	/**
	 * Конструктор класса
	 *
	 * @param Init $diafan
	 * @return \Model
	 */
	public function __construct(&$diafan, $current_module = '')
	{
		$this->diafan = &$diafan;
		$this->current_module = $current_module;
	}

	/**
	 * Проверяет есть ли доступ к элементу/категории модуля
	 *
	 * @param integer $element_id номер элемента
	 * @param integer $cat_id номер категории
	 * @param string $module_name модуль
	 * @return boolean
	 */
	protected function access($element_id, $cat_id = 0, $module = '')
	{
		if (!$module)
		{
			$module = $this->diafan->module;
		}

		if (! $this->diafan->_user->id)
		{
			return false;
		}

		return (bool)DB::query_result("SELECT role_id FROM {access} WHERE element_id=%d AND cat_id=%d AND module_name='%s' AND role_id=%d LIMIT 1", $element_id, $cat_id, $module, $this->diafan->_user->role_id);
	}

	/**
	 * Форматирует дату в соответствии с конфигурацией модуля
	 * 
	 * @param integer $date дата в формате UNIX
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $site_id номер страницы сайта
	 * @return string
	 */
	protected function format_date($date, $module_name = '', $site_id = 0)
	{
		$months_array = array(
			'01' => $this->diafan->_('января'),
			'02' => $this->diafan->_('февраля'),
			'03' => $this->diafan->_('марта'),
			'04' => $this->diafan->_('апреля'),
			'05' => $this->diafan->_('мая'),
			'06' => $this->diafan->_('июня'),
			'07' => $this->diafan->_('июля'),
			'08' => $this->diafan->_('августа'),
			'09' => $this->diafan->_('сентября'),
			'10' => $this->diafan->_('октября'),
			'11' => $this->diafan->_('ноября'),
			'12' => $this->diafan->_('декабря')
		);
		$week_array = array(
			'1' => $this->diafan->_('понедельник'),
			'2' => $this->diafan->_('вторник'),
			'3' => $this->diafan->_('среда'),
			'4' => $this->diafan->_('четверг'),
			'5' => $this->diafan->_('пятница'),
			'6' => $this->diafan->_('суббота'),
			'0' => $this->diafan->_('воскресенье')
		);
		if (!$module_name)
		{
			$module_name = $this->diafan->module;
		}
		if (!$site_id)
		{
			$site_id = $this->diafan->cid;
		}

		$config_format = $this->diafan->configmodules("format_date", $module_name, $site_id);

		switch ($config_format)
		{
			case 1:
			return date("d ", $date).$months_array[date("m", $date)].date(" Y ", $date).' '.$this->diafan->_('г.');
		
			case 2:
			return date("d ", $date).$months_array[date("m", $date)];
		
			case 3:
			return date("d ", $date).$months_array[date("m", $date)].date(" Y, ", $date).$week_array[date("w", $date)];
		
			case 4:
			return '';
		
			case 5:
			return $this->format_date_5($date);
		
			default:
			return date("d.m.Y", $date);
		}
	}

	/**
	 * Получает имя, никнейм и аватар пользователя сайта
	 * 
	 * @param integer $author номер пользователя сайта
	 * @return array
	 */
	protected function get_author($author)
	{
		if (!$author)
			return $this->diafan->_('Гость');

		global $authors;
		if (empty($authors[$author]))
		{
			if ($this->diafan->_user->id != $author)
			{
			$authors[$author] = DB::fetch_array(DB::query("SELECT fio, name FROM {users} WHERE act='1' AND trash='0' AND id=%d", $author));
			if ($this->diafan->configmodules("avatar", "users"))
			{
				$authors[$author]["avatar"] = file_exists(ABSOLUTE_PATH.USERFILES.'/avatar/'.$authors[$author]["name"].'.png') ? BASE_PATH.'userfiles/avatar/'.$authors[$author]["name"].'.png' : '';
				$authors[$author]["avatar_width"] = $this->diafan->configmodules("avatar_width", "users");
				$authors[$author]["avatar_height"] = $this->diafan->configmodules("avatar_height", "users");
			}
			}
			else
			{
			$avatar = file_exists(ABSOLUTE_PATH.USERFILES.'/avatar/'.$this->diafan->_user->name.'.png') ? BASE_PATH.USERFILES.'/avatar/'.$this->diafan->_user->name.'.png?'.date('H') : '';
			$authors[$author] = array(
				"fio" => $this->diafan->_user->fio,
				"name" => $this->diafan->_user->name
			);
			if ($this->diafan->configmodules("avatar", "users"))
			{
				$authors[$author]["avatar"] = $avatar;
				$authors[$author]["avatar_width"] = $this->diafan->configmodules("avatar_width", "users");
				$authors[$author]["avatar_height"] = $this->diafan->configmodules("avatar_height", "users");
			}
			}

			$user_page = $this->diafan->_route->module("users", true);
			
			if(! empty($user_page) && array_key_exists('name', $authors[$author]))
			{
				$authors[$author]["user_page"] = BASE_PATH_HREF.$user_page.'?'.$authors[$author]['name'];
			}

		}
		return $authors[$author];
	}

	/**
	 * Осуществляет "Умное" форматирование даты
	 * 
	 * @param integer $date дата в формет UNIX
	 * @return string
	 */
	private function format_date_5($date)
	{
		if (!$date)
			return '';

		$months_array = array(
			'01' => $this->diafan->_('января'),
			'02' => $this->diafan->_('февраля'),
			'03' => $this->diafan->_('марта'),
			'04' => $this->diafan->_('апреля'),
			'05' => $this->diafan->_('мая'),
			'06' => $this->diafan->_('июня'),
			'07' => $this->diafan->_('июля'),
			'08' => $this->diafan->_('августа'),
			'09' => $this->diafan->_('сентября'),
			'10' => $this->diafan->_('октября'),
			'11' => $this->diafan->_('ноября'),
			'12' => $this->diafan->_('декабря')
		);
		$week_array = array(
			'1' => $this->diafan->_('понедельник'),
			'2' => $this->diafan->_('вторник'),
			'3' => $this->diafan->_('среда'),
			'4' => $this->diafan->_('четверг'),
			'5' => $this->diafan->_('пятница'),
			'6' => $this->diafan->_('суббота'),
			'0' => $this->diafan->_('воскресенье')
		);
		if (time() - $date < 3600)
		{
			$min = round((time() - $date) / 60);
			if ($min < 2)
			{
			return $this->diafan->_('1 минуту назад');
			}
			if ($min % 10 == 1 && $min > 20)
			{
			return $this->diafan->_('%s минуту назад', false, $min);
			}
			if ($min % 10 < 4 && ($min > 20 || $min < 10))
			{
			return $this->diafan->_('%s минуты назад', false, $min);
			}
			return $this->diafan->_('%s минут назад', false, $min);
		}

		if ($date >= mktime(0, 0, 0, date("m"), date("d"), date("Y")))
		{
			return  $this->diafan->_('Сегодня').', '.date("H:i", $date);
		}

		if ($date >= mktime(0, 0, 0, date("m"), date("d"), date("Y")) - 86400)
		{
			return $this->diafan->_('Вчера').', '.date("H:i", $date);
		}

		if ($date >= time() - 86400 * 30)
		{
			return date("d ", $date).$months_array[date("m", $date)].', '.$week_array[date("w", $date)];
		}

		if (date("Y", $date) == date("Y"))
		{
			return date("d ", $date).$months_array[date("m", $date)];
		}
		return date("d ", $date).$months_array[date("m", $date)].date(" Y ", $date).' '.$this->diafan->_('г.');
	}

	/**
	 * Выдает ошибку о неправильном заполнении формы
	 * 
	 * @param string $module название модуля
	 * @param string $field название поля
	 * @return string
	 */
	protected function get_error($module = '', $field = '')
	{
		if (empty($_SERVER["HTTP_REFERER"]))
			return '';

		$ref = parse_url($_SERVER["HTTP_REFERER"]);

		if ($ref["host"] != $_SERVER["HTTP_HOST"])
			return '';

		$field = 'mess'.($field ? '-'.$field : '');
		if (empty($_GET["error"]) || $_GET["error"] != $module || empty($_GET[$field]))
			return '';

		return $this->diafan->get_param($_GET, $field, '', 1);
	}

	/**
	 * Получает массив полей формы
	 * 
	 * @param array $config настройки функции: module модуль, table таблица, where условие для SQL-запроса
	 * @return array
	 */
	public function get_params($config)
	{
		if (! empty($config["module"]))
		{
			$module = $config["module"];
		}
		else
		{
			$module = $this->diafan->module;
		}
		if (! empty($config["table"]))
		{
			$table =  $config["table"];
		}
		else
		{
			$table =  $module;
		}

		$where = "";
		if (! empty($config["where"]))
		{
			$where = " AND ".$config["where"];
		}

		$cache_meta = array(
			"name" => $module."_param",
			"lang_id" => _LANG,
			"where" => $where
		);
		if (! $rows_param = $this->diafan->_cache->get($cache_meta, $module))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required, [text], config FROM {".$table."_param} WHERE trash='0'"
			                    .$where." ORDER BY sort ASC");
	
			while ($row = DB::fetch_array($result))
			{
				if ($row["type"] == 'select' || $row["type"] == 'multiple' || $row["type"] == 'checkbox')
				{
					$result_select = DB::query("SELECT [name], id, value FROM {".$table."_param_select} WHERE param_id=%d"
					                           ." ORDER BY sort ASC", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][] = $row_select;
						$row["select_values"][$row["type"] == 'checkbox' ? $row_select["value"] : $row_select["id"]] = $row_select["name"];
					}
				}
				if($row["type"] == 'attachments')
				{
					$config = unserialize($row["config"]);
					$row["max_count_attachments"] = ! empty($config["max_count_attachments"]) ? $config["max_count_attachments"] : 0;
					$row["attachments_access_admin"] = ! empty($config["attachments_access_admin"]) ? $config["attachments_access_admin"] : 0;
					$row["attachment_extensions"] = ! empty($config["attachment_extensions"]) ? $config["attachment_extensions"] : '';
					$row["use_animation"] = ! empty($config["use_animation"]) ? true : false;
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, $module);
		}
		return $rows_param;
	}

	/**
	 * Генерирует данные для навигации "Хлебные крошки"
	 *
	 * @param string $module модуль
	 * @return array
	 */
	protected function get_path($module)
	{
		$path = '';
		if ($this->diafan->cat || $this->diafan->param || $this->diafan->show || ! empty($_GET["action"])
			|| $this->diafan->year)
		{
			$path[] = array("link" => $this->diafan->_route->link($this->diafan->cid), "name" => $this->diafan->name);
		}

		if ($this->diafan->cat)
		{
			$parents = $this->diafan->get_parents($this->diafan->cat, $module.'_category');
			$parents[] = $this->diafan->cat;

			if (!$this->diafan->show && !isset($_GET["catid"]))
			{
				unset($parents[count($parents) - 1]);
			}
			if (!empty($parents))
			{
				$rparents = array();
				$result = DB::query("SELECT id, [name], site_id, parent_id FROM {".$module."_category} WHERE id IN (%s)", implode(',', $parents));
				while ($row = DB::fetch_array($result))
				{
					$rparents[$row["parent_id"]] = $row;
				}
				$i = 0;
				while(! empty($rparents[$i]))
				{
					$row = $rparents[$i];
					unset($rparents[$i]);
					$i = $row["id"];
					$path[] = array("id" => $row["id"], "link" => $this->diafan->_route->link($row["site_id"], $module, $row["id"]), "name" => $row["name"]);
				}
			}
		}
		return $path;
	}
}