<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.1
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * Sitemap
 * 
 * Карта сайта
 */
class Sitemap
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array языковые версии сайта
	 */
	public $lang;

	/**
	 * @var string перечень всех полей, указывающих на активность элемента
	 */
	public $act;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
		$this->get_languages();
		header('Content-type: application/xml'); 
		echo '<?xml version="1.0" encoding="UTF-8"?>';
		echo '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';
		$this->site_map();
		echo '</urlset>';
		
	}

	/**
	 * Получает языковые версии сайта
	 * 
	 * @return void
	 */
	private function get_languages()
	{
		$result = DB::query("SELECT id, shortname, base_site FROM {languages} ORDER BY id ASC");
		while ($row = DB::fetch_array($result))
		{
			if(! empty($base_site))
			{
				$row["base_site"] = false;
				$base_site = true;
			}
			$rows[] = $row;
			$this->act .= ", act".$row["id"];
		}
		if(empty($base_site))
		{
			$rows[0]["base_site"] = true;
		}
		foreach($rows as $row)
		{
			$this->lang[] = array("name" => (! $row["base_site"] ? $row["shortname"].'/' : ''), "id" => $row["id"]);
		}
	}

	/**
	 * Формирует URL страниц сайта
	 * 
	 * @return boolean true
	 */
	private function site_map()
	{
		$result = DB::query("SELECT id, module_name, timeedit".$this->act." FROM {site} WHERE trash='0' AND access='0' AND map_no_show='0' AND block='0'");
		while ($row = DB::fetch_array($result))
		{
			foreach ($this->lang as $l)
			{
				if (! $row["act".$l["id"]])
					continue;
				echo '<url>';
				echo '<loc>'. BASE_PATH.$l["name"].$this->diafan->_route->link($row["id"]).'</loc>';
				echo '<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>';
				echo '</url>';
			}
			if (file_exists(ABSOLUTE_PATH.'modules/'.$row["module_name"].'/'.$row["module_name"].'.sitemap.php'))
			{
				Customization::inc('modules/'.$row["module_name"].'/'.$row["module_name"].'.sitemap.php');
				$class = ucfirst($row["module_name"]).'_sitemap';
				eval($class.'::get($row["id"], $row["timeedit"]);');
				continue;
			}
		}
		return true;
	}
	
	/**
	 * Формирует URL категорий модулей
	 * 
	 * @param string $module название модуля
	 * @param integer $site_id номер страницы
	 * @return boolean true
	 */
	public function category_map($module, $site_id)
	{
		$result = DB::query("SELECT id, timeedit".$this->act." FROM {".$module."_category} WHERE trash='0' AND site_id=%d AND map_no_show='0' AND access='0'", $site_id);
		while ($row = DB::fetch_array($result))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {".$module."_category_rel} WHERE cat_id=%d", $row["id"]);
			$nastr = $this->diafan->configmodules("nastr", $module, $site_id);
			$link = $this->diafan->_route->link($site_id, $module, $row["id"]);
			$link2 = $link;
			if (! ROUTE_END)
			{
				$link2 = $link.'/';
			}
			elseif (ROUTE_END != '/')
			{
				$link2 = utf::substr($link, 0, -1 * utf::strlen(ROUTE_END)).'/';
			}
			foreach ($this->lang as $l)
			{
				if (! $row["act".$l["id"]])
					continue;
				echo '<url>';
				echo '<loc>'. BASE_PATH.$l["name"].$link.'</loc>';
				echo '<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>';
				echo '</url>';
				if ($count > $nastr)
				{
			
					for ($i = 2; $i <= ceil($count/$nastr); $i++)
					{
						echo '<url>';
						echo '<loc>'.BASE_PATH.$l["name"].$link2.'page'.$i.'/</loc>';
						echo '<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>';
						echo '</url>';
					}
					
				}
			}
		}
		return true;
	}
	
	/**
	 * Формирует URL элементов модулей, которые делятся категориями
	 * 
	 * @param string $module название модуля
	 * @param integer $site_id номер страницы
	 * @return boolean true
	 */
	public function element_map($module, $site_id)
	{
		$category = $this->diafan->configmodules("cat", $module, $site_id) ? true : false;

		$result = DB::query("SELECT id, timeedit".($category ? ", cat_id" : "").$this->act." FROM {".$module."} WHERE trash='0' AND site_id=%d AND map_no_show='0' AND access='0'", $site_id);
		while ($row = DB::fetch_array($result))
		{
			$link = $this->diafan->_route->link($site_id, $module, ($category ? $row["cat_id"] : ''), $row["id"]);
			
			foreach ($this->lang as $l)
			{
				if (! $row["act".$l["id"]])
					continue;
				echo '<url>';
				echo '<loc>'. BASE_PATH.$l["name"].$link.'</loc>';
				echo '<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>';
				echo '</url>';
			}
			
		}
		return true;
	}
}


$sitemap = new Sitemap($this);
exit;