<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.1
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * User
 * 
 * Работа с пользователями в пользовательской части
 */
class User extends Core
{
	/**
	 * @var integer номер текущего пользователя
	 */
	public $id = 0;

	/**
	 * @var string данные, хранящиеся в сессии
	 */
	public $session;

	/**
	 * @var boolean пользователь проверен по идентификационному хэшу
	 */
	public $checked;

	/**
	 * @var string идентификационный хэш
	 */
	private $hash;

	/**
	 * @var boolean пользователь является администратором
	 */
	public $admin;

	/**
	 * @var array права доступа администратора
	 */
	public $roles = array();

	/**
	 * @var array права доступа для модулей
	 */
	public $module_roles = array();

	/**
	 * @var string ошибка авторизации
	 */
	public $errauth;

	/**
	 * @var array характеристики пользователя
	 */
	private $fields = array('name', 'fio', 'mail', 'created', 'role_id', 'lang_id', 'htmleditor', 'background');

	/**
	 * @var array характеристики текущего пользователя
	 */
	private $user;

	/**
	 * Доступ к свойствам текущего пользователя
	 * 
	 * @return void
	 */
	public function __get($value)
	{
		if(in_array($value, $this->fields))
		{
			return isset($this->user->$value) ? $this->user->$value : '';
		}
	} 

	/**
	 * Определяет текущего пользователя
	 * 
	 * @param object $user данные о текущем пользователе
	 * @return boolean true
	 */
	public function set($user)
	{
		$this->user = $user;
		if (!empty($user->session))
		{
			$this->session = $user->session;
		}
		$this->id = $user->id;

		if (!empty($_POST["check_hash_user"]) && $this->id)
		{
			if ($id = DB::query_result("SELECT id FROM {sessions_hash} WHERE user_id=%d AND created>%d AND hash='%h'", $this->id, time() - 7200, $_POST["check_hash_user"]))
			{
				DB::query("DELETE FROM {sessions_hash} WHERE id=%d", $id);
				$this->checked = true;
			}
		}
		else
		{
			$this->checked = false;
		}

		$result = DB::query("SELECT rewrite, perm, type FROM {users_role_perm} WHERE role_id = %d", $this->role_id);

		while($row = DB::fetch_array($result))
		{
			switch($row['type'])
			{
				case 'admin':
					$this->roles[$row['rewrite']] = explode(',', $row['perm']);
					break;

				case 'site':
					$this->module_roles[$row['rewrite']] = explode(',', $row['perm']);
					break;
			}

			if(!empty($this->roles['all']) && $this->roles['all'][0] == 'all')
			{
				$this->admin = true;
			}
		}
		return true;
	}

	/**
	 * Генерируем идентификационный пользовательский хэш
	 * 
	 * @return string
	 */
	public function get_hash()
	{
		if($this->hash)
		{
			return $this->hash;
		}
		if ($this->id)
		{
			$pass = DB::query_result("SELECT password FROM {users} WHERE id=%d", $this->id);
			$this->hash = md5(substr($pass, mt_rand(0, 32), mt_rand(0, 32)).mt_rand(23, 567).substr($pass, mt_rand(0, 32), mt_rand(0, 32)));

			DB::query("INSERT INTO {sessions_hash} (user_id, created, hash) VALUES (%d, %d, '%h')", $this->id, time(), $this->hash);
			DB::query("DELETE FROM {sessions_hash} WHERE created<%d", time() - 7200);
			return $this->hash;
		}
		return '';
	}

	/**
	 * Определяет, что текущий пользователь не авторизован
	 * 
	 * @param string $session данные, записанные в сессии
	 * @return boolean true
	 */
	public function anonymous($session = '')
	{
		$this->session = $session;
		return true;
	}

	/**
	 * Очищает информацию о текущем пользователе
	 * 
	 * @return boolean true
	 */
	public function logout()
	{
		$this->anonymous();
		session_destroy();
		$lang = '';
		if($_GET["rewrite"]  != "logout")
		{
			$lang = str_replace('logout', '', $_GET["rewrite"]);
		}
		$this->redirect(BASE_PATH.$lang);
		return true;
	}

	/**
	 * Проверяет авторизован ли пользователь
	 * 
	 * @return boolean
	 */
	public function auth($form_values)
	{
		if (!$form_values['name'] || !$form_values['pass'])
		{
			$this->errauth = 'wrong_login_or_pass';
			return false;
		}

		if (DB::query_result("SELECT name FROM {users} WHERE trash='0' AND act='0' AND LOWER(name)=LOWER('%s')", trim($form_values['name'])))
		{
			$this->errauth = 'blocked';
			return false;
		}

		if ($this->_log())
		{
			$this->errauth = 'blocked_30_min';
			return false;
		}

		$result = DB::query("SELECT * FROM {users} u WHERE trash='0' AND act='1' AND LOWER(name)=LOWER('%s') AND password='%s'", trim($form_values['name']), encrypt(trim($form_values['pass'])));

		if (DB::num_rows($result))
		{
			$user = DB::fetch_object($result);

			$this->set($user);
			
			$rew = '';
			$rewrite = ($_REQUEST["rewrite"] ? $_REQUEST["rewrite"]."/" : '');
			if ($this->lang_id)
			{
				global $diafan;
				foreach ($diafan->languages as $language)
				{
					if($diafan->language_base_site != $language["id"])
					{
						$rewrite = preg_replace('/^'.$language["shortname"].'(\/)*/', '', $rewrite);
					}
					if($diafan->language_base_site != $this->lang_id)
					{
						$rew = (! $laguage["base_site"]) ? $language["shortname"].'/' : '';
						break;
					}
				}
			}
			Session::duration();
			$this->redirect(BASE_PATH.$rew.$rewrite);
			return true;
		}
		else
		{
			$this->update_log();
			$this->errauth = 'wrong_login_or_pass';
			return false;
		}
	}

	/**
	 * Проверяет есть ли права у пользователя на действие для модуля
	 * 
	 * @param string $action действие
	 * @param string $module_name модуль
	 * @param array $roles права пользователя
	 * @param string $type часть сайта административная/пользовательская
	 * @return boolean
	 */
	public function roles($action, $module_name = '', $roles = array(), $type = 'admin')
	{
		if (! $roles)
		{
			$roles = $type == 'admin' ? $this->roles : $this->module_roles;
			if (empty($roles))
			{
				return false;
			}
		}
		if(! $module_name)
		{
			global $diafan;
			$module_name = $diafan->module;
		}

		if(! empty($roles['all']))
		{
			return true;
		}

		if (!empty($roles[$module_name]))
		{
			if ($roles[$module_name][0] == 'all' || in_array($action, $roles[$module_name]))
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * Проверяет лог авторизации для блокировки попытки подбора паролей
	 * 
	 * @return boolean
	 */
	private function _log()
	{
		DB::query("DELETE FROM {log} WHERE created<'%d'", time());
		$ip = getenv('REMOTE_ADDR');
		$result = DB::query("SELECT `count` FROM {log} WHERE ip = '%h'", $ip);
		if (DB::num_rows($result) > 0)
		{
			$row = DB::fetch_array($result);
			if ($row['count'] > 4)
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * Обновляет лог авторизации для блокировки попытки подбора паролей
	 * 
	 * @return boolean
	 */
	private function update_log()
	{
		$ip = getenv('REMOTE_ADDR');
		$date = time() + 1800;
		$result = DB::query("SELECT `count` FROM {log} WHERE ip = '%h'", $ip);
		if (DB::num_rows($result) > 0)
		{
			DB::query("UPDATE {log} SET `count` = `count` + 1, created = '%d' WHERE ip = '%h'", $date, $ip);
		}
		else
		{
			$info = getenv('HTTP_USER_AGENT');
			DB::query("INSERT INTO {log} (ip, created, info) VALUES ('%s', '%d', '%s')", $ip, $date, $info);
		}
		return false;
	}
}
