<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.1
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_admin_category
 *
 * Редактирование категорий магазина
 */
class Shop_admin_category extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'shop_category';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'name' => array(
				'type' => 'text',
				'name' => 'Название',
				'multilang' => true,
			),
			'act' => array(
				'type' => 'checkbox',
				'name' => 'Показывать на сайте',
				'default' => true,
				'multilang' => true,
			),
			'menu' => array(
				'type' => 'module',
				'name' => 'Создать пункт в меню',
			),
			'images' => array(
				'type' => 'module',
				'name' => 'Изображения',
			),
		),
		'other_rows' => array (
			'number' => array(
				'type' => 'function',
				'name' => 'Номер',
				'help' => 'Номер элемента в БД. (для разработчиков)',
			),
			'title_meta' => array(
				'type' => 'text',
				'name' => 'Заголовок окна в браузере, тэг Title',
				'help' => 'Если не заполнен, тег title будет автоматически сформирован как "Название страницы - Название сайта"',
				'multilang' => true,
			),
			'keywords' => array(
				'type' => 'text',
				'name' => 'Ключевые слова, тэг Keywords',
				'multilang' => true,
			),
			'descr' => array(
				'type' => 'textarea',
				'name' => 'Описание, тэг Description',
				'multilang' => true,
			),
			'rewrite' => array(
				'type' => 'function',
				'name' => 'Псевдоссылка',
				'help' => 'ЧПУ (человеко-понятные урл url), адрес страницы вида: site.ru/psewdossylka/. Смотрите настройки сайта',
			),
			'access' => array(
				'type' => 'function',
				'name' => 'Доступ',
			),
			'theme' => array(
				'type' => 'function',
				'name' => 'Шаблон',
			),
			'view' => array(
				'type' => 'none',
			),
			'hr3' => 'hr',
			'parent_id' => array(
				'type' => 'select',
				'name' => 'Вложенность: принадлежит',
			),
			'site_id' => array(
				'type' => 'function',
				'name' => 'Раздел сайта',
			),
			'sort' => array(
				'type' => 'function',
				'name' => 'Сортировка: установить перед',
			),
			'map_no_show' => array(
				'type' => 'checkbox',
				'name' => 'Не показывать на карте сайта',
			),
			'show_yandex' => array(
				'type' => 'checkbox',
				'name' => 'Выгружать в Яндекс Маркет',
			),
			'hr4' => 'hr',
			'import_id' => array(
				'type' => 'text',
				'name' => 'Идентификатор для импорта',
			),
			'hr5' => 'hr',
			'comments' => array(
				'type' => 'module',
			),
			'rating' => array(
				'type' => 'module',
			),
			'anons' => array(
				'type' => 'editor',
				'name' => 'Анонс',
				'multilang' => true,
			),
			'text' => array(
				'type' => 'editor',
				'name' => 'Описание',
				'multilang' => true,
			),
			'hr_info' => 'hr',
			'admin_id' => array(
				'type' => 'function',
				'name' => 'Редактор',
			),
			'timeedit' => array(
				'type' => 'text',
				'name' => 'Время последнего изменения',
				'help' => 'Изменяется после сохранения элемента. Отдается в заголовке Last Modify',
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'parent', // содержит вложенности
		'order', // сортируется
		'menu', // используется в меню
		'act', // показать/скрыть
		'del', // удалить
		'category', // часть модуля - категории
		'category_rel', // работают вместе с таблицей {module_category_rel}
		'element_site', // делит элементы по разделам (страницы сайта, к которым прикреплен модуль)
		'trash', // использовать корзину
		'only_self', // показывать только материалы редактора, если это задано в правах пользователя
	);

	/**
	 * @var array высота поля для Визуального редактора
	 */
	public $editor_height = array (
		'anons' => 200,
		'text' => 400,
	);

	/**
	 * @var integer количество строк, выводимых на странице
	 */
	public $nastr = 100;

	/**
	 * Подготавливает конфигурацию модуля
	 * @return void
	 */
	public function prepare_config()
	{
		if (! $this->diafan->configmodules('show_yandex_category', 'shop'))
		{
			$this->diafan->variable_unset("show_yandex");
		}
	}

	/**
	 * Выводит список категорий
	 * @return void
	 */
	public function show()
	{
		$this->diafan->addnew_init('Добавить категорию');
		$this->diafan->list_row();
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return void
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->del_or_trash_where("shop_param_category_rel", "cat_id=".$del_id, $trash_id);
		$this->diafan->del_or_trash_where("shop_discount_object",    "cat_id=".$del_id, $trash_id);
	}
}