<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.1
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_admin_order
 *
 * Редактирование заказов товаров
 */
class Shop_admin_order extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'shop_order';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'number' => array(
				'type' => 'function',
				'name' => 'Заказ №',
				'help' => 'Номер элемента в БД. (для разработчиков)',
			),
			'created' => array(
				'type' => 'datetime',
				'name' => 'Дата',
				'help' => 'Вводится в формате дд.мм.гггг чч:мм',
			),
			'lang_id' => array(
				'type' => 'select',
				'name' => 'Язык интерфейса',
			),
			'hr1' => 'hr',
			'memo' => array(
				'type' => 'function',
				'name' => 'Накладная',
				'no_save' => true
			),
			'goods' => array(
				'type' => 'function',
				'name' => 'Товары',
			),
			'payment_id' => array(
				'type' => 'select',
				'name' => 'Способ оплаты',
			),
			'delivery_id' => array(
				'type' => 'select',
				'name' => 'Способ доставки',
			),
			'hr2' => 'hr',
			'user_id' => array(
				'type' => 'function',
				'name' => 'Пользователь',
			),
			'param' => array(
				'type' => 'function',
			),
			'hr3' => 'hr',
			'status_id' => array(
				'type' => 'function',
				'name' => 'Статус',
				'help' => 'При смене статуса на «В обработке», количество товаров на складе уменьшается, если в конфигурации модуля отмечено «Использовать количество товаров на складе»',
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'del', // удалить
		'datetime', // показывать дату в списке, сортировать по дате
		'trash', // использовать корзину
	);

	/**
	 * @var array списки из таблицы
	 */
	public $select = array(
		'payment_id' => array(
			"shop_payment",
			"id",
			"nameLANG",
			"",
			"-",
			"trash='0'",
		),
		'delivery_id' => array(
			"shop_delivery",
			"id",
			"nameLANG",
			"",
			"-",
			"trash='0'",
		),
	);

	/**
	 * @var array значения списков
	 */
	public $select_arr = array(
		'color' => array(
			0 => 'red',
			1 => 'blue',
			2 => 'gray',
			3 => 'darkgreen',
			4 => 'black',
		),
	);

	/**
	 * @var array выводить в списке содержание полей:
	 */
	public $config_other_row = array (
		'status_id' => 'function', 
		'summ' => 'function',
		'user_id' => 'function',
	);

	/**
	 * @var array текст для ссылки на редактирование в списке
	 */
	public $text_for_base_link = array(
		'variable' => 'id',
		'text' => 'Заказ № %d'
	);

	/**
	 * Подготавливает конфигурацию модуля
	 * @return void
	 */
	public function prepare_config()
	{
		if(count($this->diafan->languages) > 2)
		{
			foreach ($this->diafan->languages as $language)
			{
				$this->diafan->select_arr("lang_id", $language["id"], $language["name"]);
			}
		}
		else
		{
			$this->diafan->variable_unset("lang_id");
		}
		$result = DB::query("SELECT id, [name], status FROM {shop_order_status} WHERE trash='0' ORDER BY sort ASC");
		while($row = DB::fetch_array($result))
		{
			$GLOBALS["status"][$row["id"]] = $row["status"];
			$this->diafan->select_arr("status_id", $row["id"], $row["name"]);
		}
	}

	/**
	 * Выводит список заказов
	 * @return void
	 */
	public function show()
	{	
        echo '<script type="text/javascript" src="'.BASE_PATH.'modules/shop/admin/shop.admin.order.js"></script>';
		echo '<script type="text/javascript">
			last_order_id = '.DB::query_result("SELECT MAX(id) FROM {shop_order} WHERE trash='0'").';
			title = \''.$this->diafan->_('Новый заказ').'\';
			timeout = 120000;
			setTimeout("check_new_order()", timeout);
		</script>';
		foreach ($this->diafan->select_arr("status_id") as $id => $value)
		{
			$first_status = $id;
			break;
		}

		echo '<br><a href="'.BASE_PATH_HREF.'shop/order/?status='.$first_status.'" class="new_order">'.$this->diafan->_('Проверить новые заказы').'</a><br><br>';
		
		echo '<form action="'.URL.'" method="GET">
		<p>'.$this->diafan->_('Статус').': <select name="status">
		<option value="all">'.$this->diafan->_('Все').'</option>';
		foreach ($this->diafan->select_arr("status_id") as $id => $value)
		{
			echo '<option value="'.$id.'"'.(isset($_GET["status"]) && $_GET["status"] == $id ? ' selected' : '').'>'.$value.'</option>';
		}
		echo '</select></p>
		</form>';

		if (isset($_GET["status"]))
		{
			$status = (int)$this->diafan->get_param($_GET, "status", 0, 2);
			$this->diafan->where = " AND status_id='".$status."'";
			$this->diafan->get_nav = ($this->diafan->get_nav ? '&' : '?').'status='.$status;
		}
		$this->diafan->list_row();
		
		if (! $this->diafan->count)
		{
			echo '<center><b>'.$this->diafan->_('Заказов нет').'</b><br>('
			.$this->diafan->_('заказы создаются посетителями из пользовательской части сайта')
			.')</center>';
		}
	}

	/**
	 * Выводит имя заказчика в списке заказов
	 * @return string
	 */
	public function other_row_user_id($row)
	{
		if($row["user_id"])
		{
			$text = '</td><td class="comment"><a href="'.BASE_PATH_HREF.'users/edit'.$row["user_id"].'/">'.DB::title("users", $row["user_id"], "fio").'</a>';
		}
		else
		{
			$text = '</td><td class="comment">
				<div>
					<span></span>
					<table><tbody><tr><td>';
			$values = '';
	
			$result = DB::query("SELECT e.value, e.param_id, p.type, p.[name] FROM {shop_order_param_element} AS e"
								." INNER JOIN {shop_order_param} AS p ON e.param_id=p.id"
								. " WHERE e.trash='0' AND e.element_id=%d", $row["id"]);
			while ($row = DB::fetch_array($result))
			{
				if ($row["value"])
				{
					switch ($row["type"])
					{
						case 'select':
						case 'multiple':
							$row["value"] = DB::query_result("SELECT [name] FROM {shop_order_param_select} WHERE id=%d", $row["value"]);
							break;
	
						case 'checkbox':
							$v = DB::query_result("SELECT [name] FROM {shop_order_param_select} WHERE param_id=%d AND value=1", $row["param_id"]);
							if ($v)
							{
								$row["value"] = $row["name"] . ': ' . $v;
							}
							else
							{
								$row["value"] = $row["name"];
							}
							break;
					}
					$values .= ( $values ? ', ' : '' ) . $row["value"];
				}
			}
			$text .= $values . '</td></tr></tbody></table>
			</div>';
		}
		return $text;
	}

	/**
	 * Выводит сумму заказа в списке заказов
	 * @return string
	 */
	public function other_row_summ($row)
	{
		return '</td><td class="summ">'
		.($row["summ"]
		 ? $this->diafan->_shop->price_format($row["summ"]).' '.$this->diafan->configmodules("currency", "shop")
		 : '');
	}

	/**
	 * Выводит статус заказа в списке заказов
	 * @return string
	 */
	public function other_row_status_id($row)
	{
		return '<td class="status">'
		.'<span style="color:'.$this->diafan->select_arr("color", $GLOBALS["status"][$row["status_id"]]).';font-weight: bold;">'
		.$this->diafan->select_arr("status_id", $row["status_id"]).'</span></td>';
	}

	/**
	 * Редактирование поля "Номер"
	 * @return void
	 */
	public function edit_variable_number()
	{
		echo '<tr>
			<td align="right">
				'.$this->diafan->variable_name().'
			</td>
			<td style="color:#999999;font-weight:bold">
				'.$this->diafan->edit.' '.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Редактирование поля "Статус"
	 * @return void
	 */
	public function edit_variable_status_id()
	{
		echo '
		<tr>
			<td align="right">
				'.$this->diafan->variable_name().'
			</td>
			<td>';
		echo '<select name="status_id" id="order_select_status">';
		foreach ($this->diafan->select_arr("status_id") as $key => $value)
		{
			echo '<option value="'.$key.'"'.($key == $this->diafan->value ? ' selected' : '').'>'.$value.'</option>';
		}
		echo '</select>'.$this->diafan->help();
		echo '</td>
		</tr>';
	}

	/**
	 * Редактирование поля "Накладная"
	 * @return void
	 */
	public function edit_variable_memo()
	{
		echo '
		<tr valign="top">
			<td align="right">'.$this->diafan->variable_name().'</td>
			<td><a href="'.BASE_PATH.'cart/payment/non_cash/memo/'.$this->diafan->edit.'/">'.$this->diafan->_('Сформировать').'</a></td>
		</tr>';
	}

	/**
	 * Редактирование поля "Товары"
	 * @return void
	 */
	public function edit_variable_goods()
	{
		$summ = 0;
		$count = 0;
		$cat = 0;
		echo '
		<tr valign="top">
			<td align="right">'.$this->diafan->variable_name().'</td>
			<td>
                <script type="text/javascript" src="'.BASE_PATH.'modules/shop/admin/shop.admin.order.js"></script>
				<table class="border" cellspacing="0">
					<tr id="tr_first">
						<td id="first"></td>
						<td>'.$this->diafan->_('Наименование товара').'</td>
						<td>'.$this->diafan->_('Количество').'</td>
						<td>'.$this->diafan->_('Цена').'</td>
						<td>'.$this->diafan->_('Скидка').'</td>
						<td>'.$this->diafan->_('Сумма').'</td>
                                                    
					</tr>';
		$format_price = intval($this->diafan->configmodules("format_price_1", "shop"));

		$result = DB::query(
				"SELECT g.*, s.name".$this->diafan->language_base_site." AS name_good, s.article, s.cat_id, c.name".$this->diafan->language_base_site." AS name_cat FROM {shop_order_goods} AS g"
				." INNER JOIN {shop} AS s ON g.good_id=s.id"
				." LEFT JOIN {shop_category} AS c ON s.cat_id=c.id"
				." WHERE g.order_id=%d ORDER by c.sort ASC",
				$this->diafan->values["id"]
			); 
		while ($row = DB::fetch_array($result))
		{
			$depend = '';
			$result_p = DB::query("SELECT * FROM {shop_order_goods_param} WHERE order_good_id=%d", $row["id"]); 
			while ($row_p = DB::fetch_array($result_p))
			{
				$param_name  = DB::query_result("SELECT [name] FROM {shop_param} WHERE id=%d", $row_p["param_id"]);
				$param_value = DB::query_result("SELECT s.[name] FROM {shop_param_select} as s WHERE s.id=%d AND s.param_id=%d", $row_p["value"], $row_p["param_id"]);
				$depend .= ($depend ? ', ' : '').$param_name.': '.$param_value;
			}
			if($row["discount_id"])
			{
				if(empty($discounts[$row["discount_id"]]))
				{
					$d = DB::fetch_array(DB::query("SELECT discount, deduction FROM {shop_discount} WHERE id=%d", $row["discount_id"]));
					$discounts[$row["discount_id"]] = $d["discount"] ? $d["discount"].'%' : $d["deduction"].' '.$this->diafan->configmodules("currency", "shop");
				}
				$row["discount"] = $discounts[$row["discount_id"]];
			}
			if($cat != $row["cat_id"])
			{
				echo '<tr><td></td><td colspan="6"><b>'.$row["name_cat"].'</b></tr>';
				$cat = $row["cat_id"];
			}
			$img = DB::query_result("SELECT name FROM {images} WHERE element_id=%d AND module_name='shop' AND trash='0' ORDER BY sort ASC LIMIT 1", $row["good_id"]);
			
			echo '
			<tr class="tr_good">
				<td id="first">'.($img ? '<img src="'.BASE_PATH.USERFILES.'/small/'.$img.'">' : '').'</td>
				<td><a href="'.BASE_PATH_HREF.'shop/edit'.$row["good_id"].'/">'.$row["name_good"].' '.$depend.' '.$row["article"].'</a></td>
				<td align="center"><input type="text" name="count_goods'.$row["id"].'" value="'.$row["count_goods"].'" size="2"></td>
				<td align="right"><input type="text" name="price_goods'.$row["id"].'" value="'.number_format($row["price"], $format_price, ".", "").'" size="4"></td>
				<td>'.($row["discount_id"] ? '<a href="'.BASE_PATH_HREF.'shop/discount/edit'.$row["discount_id"].'/">'.$row["discount"].'</a>' : '').'</td>
				<td align="right">'.$this->diafan->_shop->price_format($row["price"] * $row["count_goods"]).'</td>
				<td><a href="javascript:void(0)" confirm="'.$this->diafan->_('Вы действительно хотите удалить запись?').'" class="delete_order_good" title="'.$this->diafan->_('Удалить').'">
				<img src="'.BASE_PATH.'adm/img/delete.png" width="13" height="13" alt="'.$this->diafan->_('Удалить').'"></a></td>
</tr>';
			$summ += $row["price"] * $row["count_goods"];
			$count += $row["count_goods"];
		}
		echo '<tr class="tr_good">
				<td id="first">&nbsp;</td>
				<td colspan="4">
				<a href="javascript:void(0)" class="order_good_plus" title="' . $this->diafan->_('Добавить') . '"><img src="' . BASE_PATH . 'adm/img/add_new.png" width="14" height="14" alt="' . $this->diafan->_('Добавить') . '"></a>
				<div class="hide" id="order_goods_container"></div></td>
		</tr>';
		$result = DB::query("SELECT a.id, a.[name], a.price, a.amount, s.id AS sid, s.summ FROM {shop_additional_cost} AS a LEFT JOIN {shop_order_additional_cost} AS s ON s.additional_cost_id=a.id AND s.order_id=%d WHERE a.trash='0'", $this->diafan->values["id"]); 
		while ($row = DB::fetch_array($result))
		{
			if($row["sid"])
			{
				$row['price'] = $row['summ'];
			}
			else
			{
				if (! empty($row['amount']))
				{
					if ($row['amount'] < $summ)
					{
						$row['price'] = 0;
					}
				}
			}
		    echo '<tr class="tr_good">
				<td id="first">&nbsp;</td>
				<td colspan="4">'.$row["name"].'</td>
				<td align="right" class="order_good_price"><input type="text" name="summ_additional_cost'.$row["id"].'" value="'.$row["price"].'" size="4"></td>
				<td><input name="additional_cost_id'.$row["id"].'" value="1" type="checkbox" '.($row["sid"] ? ' checked' : '').'></td>
			</tr>';
		}
		if ($this->diafan->values["delivery_id"])
		{
			$delivery_name = DB::query_result("SELECT [name] FROM {shop_delivery} WHERE id=%d", $this->diafan->values["delivery_id"]);
		    echo '<tr>
				<td id="first">&nbsp;</td>
				<td colspan="4">'.$delivery_name.'</td>
				<td align="right" class="order_good_price">'.$this->diafan->_shop->price_format($this->diafan->values["delivery_summ"]).'</td>
			</tr>';
		}
		echo '
			<tr>
				<td id="first">&nbsp;</td>
				<td>'.$this->diafan->_('ИТОГО').'</td>
				<td align="center" class="order_good_count">'.$count.'</td>
				<td>&nbsp;</td>
				<td>&nbsp;</td>
				<td align="right" class="order_good_price">'.$this->diafan->_shop->price_format($this->diafan->values["summ"]).'</td>
			</tr>
		</table>
		'.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Сохранение поля "Товары"
	 * @return void
	 */
	public function save_variable_goods()
	{
		$summ = 0;
		$result = DB::query("SELECT * FROM {shop_order_goods} WHERE order_id=%d", $this->diafan->save); 
		while ($row = DB::fetch_array($result))
		{
			if(empty($_POST["count_goods".$row["id"]]))
			{
				DB::query("DELETE FROM {shop_order_goods} WHERE id=%d", $row["id"]);
			}
			else
			{
				if ($_POST["count_goods".$row["id"]] != $row["count_goods"])
				{
					DB::query("UPDATE {shop_order_goods} SET count_goods=%d WHERE id=%d", $_POST["count_goods".$row["id"]], $row["id"]);
				}
				if ($_POST["price_goods".$row["id"]] != $row["price"])
				{
					DB::query("UPDATE {shop_order_goods} SET price=%f WHERE id=%d", $_POST["price_goods".$row["id"]], $row["id"]);
				}

				$summ += $_POST["price_goods".$row["id"]] * $_POST["count_goods".$row["id"]];
			}
		}
		$result = DB::query("SELECT a.id, s.id AS sid, s.summ FROM {shop_additional_cost} AS a LEFT JOIN {shop_order_additional_cost} AS s ON s.additional_cost_id=a.id AND s.order_id=%d WHERE a.trash='0'", $this->diafan->save); 
		while ($row = DB::fetch_array($result))
		{
			if($row["sid"])
			{
				if(empty($_POST['additional_cost_id'.$row["id"]]))
				{
					DB::query("DELETE FROM {shop_order_additional_cost} WHERE id=%d", $row["sid"]);
				}
				elseif($row['summ'] != $_POST['summ_additional_cost'.$row["id"]])
				{
					DB::query("UPDATE {shop_order_additional_cost} SET summ=%f WHERE id=%d", $_POST['summ_additional_cost'.$row["id"]], $row["sid"]);
				}
			}
			else
			{
				if(! empty($_POST['additional_cost_id'.$row["id"]]))
				{
					DB::query("INSERT INTO {shop_order_additional_cost} (additional_cost_id, summ, order_id) VALUES (%d, %f, %d)", $row["id"], $_POST['summ_additional_cost'.$row["id"]], $this->diafan->save);
				}
			}
		}
		
		$result = DB::query("SELECT a.id, a.[name], s.summ, s.id AS a_id FROM {shop_additional_cost} AS a INNER JOIN {shop_order_additional_cost} AS s ON s.additional_cost_id=a.id WHERE s.order_id=%d", $this->diafan->save); 
		while ($row = DB::fetch_array($result))
		{
			if(! isset($_POST['summ_additional_cost'.$row["id"]]))
			{
				DB::query("DELETE FROM {shop_order_additional_cost} WHERE id=%d", $row["a_id"]);
			}
			else
			{
				if($_POST['summ_additional_cost'.$row["id"]] != $row["summ"])
				{
					DB::query("UPDATE {shop_order_additional_cost} SET summ=%f WHERE id=%d", $_POST['summ_additional_cost'.$row["id"]], $row["a_id"]);
				}
				$summ += $_POST['summ_additional_cost'.$row["id"]];
			}
		}
		DB::query("UPDATE {shop_order} SET summ=%f+delivery_summ WHERE id=%d", $summ, $this->diafan->save);
	}

	/**
	 * Сохранение поля "Способ доставки"
	 * @return void
	 */
	public function save_variable_delivery_id()
	{
		$result = DB::query("SELECT id, price, amount FROM {shop_delivery}  WHERE [act]='1' AND trash='0'AND id=%d", $_POST["delivery_id"]);
		if ($row = DB::fetch_array($result))
		{
			if (! empty($row['amount']))
			{
				if ($row['amount'] < DB::query_result("SELECT summ FROM {shop_order} WHERE id=%d", $this->diafan->save))
				{
					$row['price'] = 0;
				}
			}
			$delivery_summ = $row["price"];
			$delivery_id = $row["id"];
		}
		DB::query("UPDATE {shop_order} SET summ=summ-delivery_summ+%f, delivery_summ=%f, delivery_id=%d WHERE id=%d", $delivery_summ, $delivery_summ, $delivery_id, $this->diafan->save);
	}

	/**
	 * Сохранение поля "Статус",
     * отправка ссылок на купленные файлы при необходимости
     * 
	 * @return void
	 */
	public function save_variable_status_id()
	{
		if($this->diafan->oldrow["status_id"] == $_POST["status_id"])
			return;

		$status = DB::query_result("SELECT status FROM {shop_order_status} WHERE id=%d", $_POST["status_id"]);
		if($status == 1)
		{
			$this->diafan->_shop->order_pay($this->diafan->save);
		}
		else
		{
			$this->diafan->_shop->order_send_mail_change_status($this->diafan->save, $_POST["status_id"]);
		}

        $this->diafan->set_query("status_id=%d");
		$this->diafan->set_value($_POST["status_id"]);

        $this->diafan->set_query("status='%d'");
		$this->diafan->set_value($status);
    }
    
    
	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return boolean true
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->del_or_trash_where("shop_order_goods", "order_id=".$del_id, $trash_id);
		$this->diafan->del_or_trash_where("shop_order_goods_param", "order_good_id IN (SELECT id FROM {shop_order_goods} WHERE order_id=".$del_id.")", $trash_id);
		$this->diafan->del_or_trash_where("shop_order_param_element", "element_id=".$del_id, $trash_id);
	}
}