<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.1
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_ajax
 *
 * Обработка запроса при добавлении товара в корзину
 */
class Shop_ajax extends Ajax
{

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (empty($_POST['module']) || $_POST['module'] != 'shop' || empty($_POST['action']))
		{
			return false;
		}
	
		switch ($_POST['action'])
		{
			case 'buy':
			return $this->buy();
	
			case 'wish':
			return $this->wish();
	
			case 'wait':
			return $this->wait();
	
			case 'add_coupon':
			return $this->add_coupon();
	
			case 'compare_goods':
			return $this->compare_goods();
	
			case 'compare_delete_goods':
			return $this->compare_delete_goods();
		}
		return false;
	}

	/**
	 * Добавляет товар в корзину
	 * 
	 * @return boolean
	 */
	private function buy()
	{
		if (empty($_POST['good_id']) || $this->diafan->configmodules('security_user') && ! $this->diafan->_user->id)
		{
			return false;
		}
		$good_id = $this->diafan->get_param($_POST, 'good_id', 0, 2);
		$this->tag = 'shop'.$good_id;

		$row = DB::fetch_array(DB::query("SELECT id, is_file FROM {shop} WHERE id=%d AND trash='0' AND [act]='1'", $good_id));

		if (empty($row['id']))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$params = array();

		$result = DB::query(
				"SELECT p.[name], p.id FROM {shop_param} AS p"
				." INNER JOIN {shop_param_element} AS e ON e.element_id=%d AND e.param_id=p.id"
				." WHERE p.`type`='multiple' AND p.required='1' GROUP BY p.id",
				$good_id
			);
		while ($row_param = DB::fetch_array($result))
		{
			if (empty($_POST["param".$row_param["id"]]))
			{
				$this->result["errors"][0] = $this->diafan->_('Пожалуйста, выберите', false).' '.$row_param["name"];
				$this->send_errors();
				return true;
			}
			$params[$row_param["id"]] = $this->diafan->get_param($_POST, "param".$row_param["id"], 0, 2);
		}

		$count = $this->diafan->get_param($_POST, "count", 1, 2);
		$count = $count > 0 ? $count : 1;

		$count_good = $this->diafan->_cart->get($good_id, $params, "count");
		$count_good += $count;

		$cart = array(
				"count" => $count_good,
				"is_file" => $row['is_file'],
			);

		if($err = $this->diafan->_cart->set($cart, $good_id, $params))
		{
			$this->result["errors"][0] = $err;
			return $this->send_errors();	
		}
		$this->diafan->_cart->recalc();
		$this->diafan->_cart->write();

		DB::query("UPDATE {shop} SET counter_buy=counter_buy+1 WHERE id='%d'", $good_id);

		$this->result["errors"][0] = $this->diafan->_('В <a href="%s">корзине</a> %s шт.', false, BASE_PATH_HREF.$this->diafan->_route->module("cart", true), $count_good);

		$cart_tpl = array(
				"count" => $this->diafan->_cart->get_count(),
				"summ" => $this->diafan->_shop->price_format($this->diafan->_cart->get_summ_discount()),
				"currency" => $this->diafan->configmodules("currency", "shop")
			);
		$this->result["target"] = "#show_cart";
		$this->result["data"] = $this->diafan->_tpl->get('info', 'cart', $cart_tpl);

		return $this->send_errors();
	}

	/**
	 * Добавляет товар в список пожеланий
	 * 
	 * @return boolean
	 */
	private function wish()
	{
		if (empty($_POST['good_id']) || $this->diafan->configmodules('security_user') && ! $this->diafan->_user->id)
		{
			return false;
		}
		$good_id = $this->diafan->get_param($_POST, 'good_id', 0, 2);

		$row = DB::fetch_array(DB::query("SELECT id, is_file FROM {shop} WHERE id=%d AND trash='0' AND [act]='1'", $good_id));

		if (empty($row['id']))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$params = array();

		$result = DB::query(
				"SELECT p.[name], p.id FROM {shop_param} AS p"
				." INNER JOIN {shop_param_element} AS e ON e.element_id=%d AND e.param_id=p.id"
				." WHERE p.`type`='multiple' AND p.required='1' GROUP BY p.id",
				$good_id
			);
		while ($row_param = DB::fetch_array($result))
		{
			if (empty($_POST["param".$row_param["id"]]))
			{
				$this->result["errors"][0] = $this->diafan->_('Пожалуйста, выберите', false).' '.$row_param["name"];
				$this->send_errors();
				return true;
			}
			$params[$row_param["id"]] = $this->diafan->get_param($_POST, "param".$row_param["id"], 0, 2);
		}

		$count = $this->diafan->get_param($_POST, "count", 1, 2);
		$count = $count > 0 ? $count : 1;

		$count_good = $this->diafan->_wishlist->get($good_id, $params, "count");
		$count_good += $count;

		$wishlist = array(
				"count" => $count_good,
				"is_file" => $row['is_file'],
			);
		if($err = $this->diafan->_wishlist->set($wishlist, $good_id, $params))
		{
			$this->result["errors"][0] = $err;
			return $this->send_errors();	
		}
		$this->diafan->_wishlist->write();


		$this->result["errors"][0] = $this->diafan->_('Товар добавлен в <a href="%s">отложенные</a>', false, BASE_PATH_HREF.$this->diafan->_route->module("wishlist", true));

		return $this->send_errors();
	}

	/**
	 * Добавляет товар в список ожиданий
	 * 
	 * @return boolean
	 */
	private function wait()
	{
		if (empty($_POST['good_id']) || $this->diafan->configmodules('security_user') && ! $this->diafan->_user->id)
		{
			return false;
		}
		$good_id = $this->diafan->get_param($_POST, 'good_id', 0, 2);

		$row = DB::fetch_array(DB::query("SELECT id FROM {shop} WHERE id=%d AND trash='0' AND [act]='1'", $good_id));

		if (empty($row['id']))
		{
			$this->result["errors"]["waitlist"] = 'ERROR';
			return $this->send_errors();
		}
		if(empty($_POST["mail"]))
		{
			$this->result["errors"]["waitlist"] = $this->diafan->_('Пожалуйста, выберите', false).' '.$row_param["name"];
			$this->send_errors();
			return true;
		}
		Customization::inc('includes/validate.php');
		$mes = Validate::mail($_POST["mail"]);
		if ($mes)
		{
			$this->result["errors"]["waitlist"] = $this->diafan->_($mes);
			$this->send_errors();
			return true;
		}

		$params = array();

		$result = DB::query(
				"SELECT p.[name], p.id, e.value".$this->diafan->language_base_site." AS value FROM {shop_param} AS p"
				." INNER JOIN {shop_param_element} AS e ON e.element_id=%d AND e.param_id=p.id"
				." WHERE p.`type`='multiple' AND p.required='1' GROUP BY p.id",
				$good_id
			);
		while ($row_param = DB::fetch_array($result))
		{
			if (empty($_POST["param".$row_param["id"]]))
			{
				$this->result["errors"]["waitlist"] = $this->diafan->_('Пожалуйста, выберите', false).' '.$row_param["name"];
				$this->send_errors();
				return true;
			}
			if($row_param["value"])
			{
				$params[$row_param["id"]] = $this->diafan->get_param($_POST, "param".$row_param["id"], 0, 2);
			}
		}
		asort($params);
		$params = serialize($params);
		if($id = DB::query_result("SELECT id FROM {shop_waitlist} WHERE trash='0' AND good_id=%d AND mail='%h' AND param='%s'", $good_id, $_POST["mail"], $params))
		{
			DB::query("UPDATE {shop_waitlist} SET created=%d WHERE id=%d", time(), $id);
		}
		else
		{
			DB::query("INSERT INTO {shop_waitlist} (good_id, mail, param, created, user_id, lang_id) VALUES (%d,  '%h', '%s', %d, %d, %d)", $good_id, $_POST["mail"], $params, time(), $this->diafan->_user->id, _LANG);
		}

		$this->result["errors"]["waitlist"] = $this->diafan->_('Спасибо! Мы уведомим Вас когда товар поступит на склад.', false);

		return $this->send_errors();
	}

	/**
	 * Активирует купон
	 * 
	 * @return boolean
	 */
	private function add_coupon()
	{
		if (!$this->diafan->_user->id)
		{
			return false;
		}
		if (empty($_POST["coupon"]))
		{
			$this->result["errors"][0] = $this->diafan->_('Вы ввели ошибочный код купона', false);
			return $this->send_errors();
		}
		$discount_id = DB::query_result("SELECT id FROM {shop_discount} WHERE coupon='%h' AND user_id=0 AND trash='0' AND act='1'", $_POST["coupon"]);
		if (!$discount_id)
		{
			$this->result["errors"][0] = $this->diafan->_('Вы ввели ошибочный код купона', false);
		}
		else
		{
			DB::query("UPDATE {shop_discount} SET user_id=%d WHERE id=%d", $this->diafan->_user->id, $discount_id);
			$this->diafan->_shop->price_calc(0, $discount_id);
			$this->result["redirect"] = $_SERVER['HTTP_REFERER'];
		}
		return $this->send_errors();
	}

	/**
	 * Добавляет/удаляет товар для сравнения
	 * 
	 * @return boolean
	 */
	private function compare_goods()
	{
		if (empty($_POST['shop_compare_id']))
		{
			return false;
		}
		$id = $this->diafan->get_param($_POST, "shop_compare_id", 0, 2);
		$site_id = $this->diafan->cid;
	
		if (empty($_POST['shop_compare_add']))
		{
			if (isset($_SESSION['compare_list'][$site_id][$id]))
			{
				unset($_SESSION['compare_list'][$site_id][$id]);
			}
		}
		else
		{
			$_SESSION['compare_list'][$site_id][$id] = 1;
			$this->result['shop_compare_add'] = 1;
		}
		$this->result['shop_compare_id'] = $id;
		$this->result['shop_compare_input'] = '<input type="hidden" name="ids[]" value="'.$id.'" class="shop_compared_goods_'.$id.'">';
		if (!empty($_POST["ajax"]))
		{
			include_once ABSOLUTE_PATH.'plugins/json.php';
			echo to_json($this->result);
		}
		else
		{
			$this->diafan->redirect($_SERVER['HTTP_REFERER']);
		}
		return true;
	}

	/**
	 * Очищает список сравнения
	 * 
	 * @return boolean
	 */
	private function compare_delete_goods()
	{
		if (isset($_SESSION['compare_list'][$this->diafan->cid]))
		{
			unset($_SESSION['compare_list'][$this->diafan->cid]);
		}
		$this->result['redirect'] = BASE_PATH_HREF.$this->diafan->_route->link($this->diafan->cid).'?action=compare';
		if (!empty($_POST["ajax"]))
		{
			include_once ABSOLUTE_PATH.'plugins/json.php';
			echo to_json($this->result);
		}
		else
		{
			$this->diafan->redirect($this->result['redirect']);
		}
		return true;
	}
}